#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sem_post sem_post_1-1.run-test sem_post_1-2.run-test sem_post_2-1.run-test sem_post_4-1.run-test sem_post_5-1.run-test sem_post_6-1.run-test sem_post_8-1.run-test
