#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setdetachstate pthread_attr_setdetachstate_1-1.run-test pthread_attr_setdetachstate_1-2.run-test pthread_attr_setdetachstate_2-1.run-test pthread_attr_setdetachstate_3-1.sh pthread_attr_setdetachstate_4-1.run-test
