#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_rwlockattr_destroy pthread_rwlockattr_destroy_1-1.run-test pthread_rwlockattr_destroy_2-1.run-test pthread_rwlockattr_destroy_3-1.sh
