#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cond_broadcast pthread_cond_broadcast_1-1.run-test pthread_cond_broadcast_1-2.run-test pthread_cond_broadcast_2-1.run-test pthread_cond_broadcast_2-2.run-test pthread_cond_broadcast_2-3.run-test pthread_cond_broadcast_4-1.run-test pthread_cond_broadcast_4-2.run-test
