#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_getpshared pthread_mutexattr_getpshared_1-1.run-test pthread_mutexattr_getpshared_1-2.run-test pthread_mutexattr_getpshared_1-3.run-test pthread_mutexattr_getpshared_2-1.sh pthread_mutexattr_getpshared_3-1.run-test
