#! /bin/sh
#
# Test that sched_setscheduler() function return a value of -1 when it fails to
# complete successfully.
#
# This is tested implicitly via assertions 19, 20 and 21.

echo "Tested implicitly via assertions 19, 20 and 21."
exit 0
