#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutex_unlock pthread_mutex_unlock_1-1.run-test pthread_mutex_unlock_2-1.run-test pthread_mutex_unlock_3-1.run-test pthread_mutex_unlock_5-1.run-test pthread_mutex_unlock_5-2.run-test
