#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_condattr_setpshared pthread_condattr_setpshared_1-1.run-test pthread_condattr_setpshared_1-2.run-test pthread_condattr_setpshared_2-1.run-test
