#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_join pthread_join_1-1.run-test pthread_join_1-2.run-test pthread_join_2-1.run-test pthread_join_3-1.run-test pthread_join_4-1.run-test pthread_join_5-1.run-test pthread_join_6-2.run-test pthread_join_6-3.run-test
