#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/munmap munmap_1-1.run-test munmap_1-2.run-test munmap_10-1.sh munmap_2-1.run-test munmap_3-1.run-test munmap_4-1.run-test munmap_7-1.sh munmap_8-1.run-test munmap_9-1.run-test
