#!/bin/sh
../../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setschedparam/speculative pthread_attr_setschedparam_speculative_3-1.run-test pthread_attr_setschedparam_speculative_3-2.run-test
