#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cleanup_push pthread_cleanup_push_1-1.run-test pthread_cleanup_push_1-2.run-test pthread_cleanup_push_1-3.run-test pthread_cleanup_push_2-1.sh
