#!/bin/sh
../../../../bin/run-tests.sh conformance/interfaces/timer_create/speculative timer_create_speculative_15-1.run-test timer_create_speculative_2-1.run-test timer_create_speculative_5-1.run-test
