#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_condattr_destroy pthread_condattr_destroy_1-1.run-test pthread_condattr_destroy_2-1.run-test pthread_condattr_destroy_3-1.run-test pthread_condattr_destroy_4-1.run-test
