#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sem_open sem_open_1-1.run-test sem_open_1-2.run-test sem_open_1-3.run-test sem_open_1-4.run-test sem_open_10-1.run-test sem_open_15-1.run-test sem_open_2-1.run-test sem_open_2-2.run-test sem_open_3-1.run-test sem_open_4-1.run-test sem_open_5-1.run-test sem_open_6-1.run-test
