#!/bin/sh
../../../../bin/run-tests.sh conformance/interfaces/mq_open/speculative mq_open_speculative_2-2.run-test mq_open_speculative_2-3.run-test mq_open_speculative_26-1.run-test mq_open_speculative_6-1.run-test
