/* include/config.h.  Generated from config.h.in by configure.  */
/* include/config.h.in.  Generated from configure.ac by autoheader.  */

/* Define to 1 if you have the <asm/ldt.h> header file. */
/* #undef HAVE_ASM_LDT_H */

/* Define to 1 if you have the __atomic_* compiler builtins */
#define HAVE_ATOMIC_MEMORY_MODEL 1

/* Define to 1 if you have __builtin___clear_cache */
#define HAVE_BUILTIN_CLEAR_CACHE 1

/* Define to 1 if you have the 'clnttcp_create' function. */
/* #undef HAVE_CLNTTCP_CREATE */

/* Define to 1 if you have the 'clone3' function. */
/* #undef HAVE_CLONE3 */

/* Define to 1 if you have the 'close_range' function. */
#define HAVE_CLOSE_RANGE 1

/* Define to 1 if you have the 'copy_file_range' function. */
#define HAVE_COPY_FILE_RANGE 1

/* Define to 1 if you have the <cpuid.h> header file. */
/* #undef HAVE_CPUID_H */

/* Define to 1 if you have the 'daemon' function. */
#define HAVE_DAEMON 1

/* Define to 1 if you have the declaration of 'IFLA_NET_NS_PID', and to 0 if
   you don't. */
#define HAVE_DECL_IFLA_NET_NS_PID 1

/* Define to 1 if you have the declaration of 'MADV_MERGEABLE', and to 0 if
   you don't. */
#define HAVE_DECL_MADV_MERGEABLE 1

/* Define to 1 if you have the declaration of 'NFTA_CHAIN_ID', and to 0 if you
   don't. */
#define HAVE_DECL_NFTA_CHAIN_ID 1

/* Define to 1 if you have the declaration of 'NFTA_VERDICT_CHAIN_ID', and to
   0 if you don't. */
#define HAVE_DECL_NFTA_VERDICT_CHAIN_ID 1

/* Define to 1 if you have the declaration of 'PR_CAPBSET_DROP', and to 0 if
   you don't. */
#define HAVE_DECL_PR_CAPBSET_DROP 1

/* Define to 1 if you have the declaration of 'PR_CAPBSET_READ', and to 0 if
   you don't. */
#define HAVE_DECL_PR_CAPBSET_READ 1

/* Define to 1 if you have the declaration of 'SEM_STAT_ANY', and to 0 if you
   don't. */
#define HAVE_DECL_SEM_STAT_ANY 1

/* Define to 1 if you have the <emmintrin.h> header file. */
/* #undef HAVE_EMMINTRIN_H */

/* Define to 1 if the system has the type 'enum kcmp_type'. */
#define HAVE_ENUM_KCMP_TYPE 1

/* Define to 1 if you have the 'epoll_pwait' function. */
#define HAVE_EPOLL_PWAIT 1

/* Define to 1 if you have the 'epoll_pwait2' function. */
#define HAVE_EPOLL_PWAIT2 1

/* Define to 1 if you have the 'execveat' function. */
#define HAVE_EXECVEAT 1

/* Define to 1 if you have the 'faccessat2' function. */
/* #undef HAVE_FACCESSAT2 */

/* Define to 1 if you have the 'fallocate' function. */
#define HAVE_FALLOCATE 1

/* Define to 1 if you have the 'fork' function. */
#define HAVE_FORK 1

/* Define to 1 if you have the 'fsconfig' function. */
#define HAVE_FSCONFIG 1

/* Define to 1 if you have the 'fsmount' function. */
#define HAVE_FSMOUNT 1

/* Define to 1 if you have the 'fsopen' function. */
#define HAVE_FSOPEN 1

/* Define to 1 if you have the 'fspick' function. */
#define HAVE_FSPICK 1

/* Define to 1 if you have the 'fstatat' function. */
#define HAVE_FSTATAT 1

/* Define to 1 if you have the <fts.h> header file. */
#define HAVE_FTS_H 1

/* Define to 1 if you have the 'getauxval' function. */
#define HAVE_GETAUXVAL 1

/* Define to 1 if you have the 'getcontext' function. */
#define HAVE_GETCONTEXT 1

/* Define to 1 if you have the 'getcpu' function. */
#define HAVE_GETCPU 1

/* Define to 1 if you have the 'getdents' function. */
/* #undef HAVE_GETDENTS */

/* Define to 1 if you have the 'getdents64' function. */
#define HAVE_GETDENTS64 1

/* Define to 1 if you have the <ifaddrs.h> header file. */
#define HAVE_IFADDRS_H 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the 'io_pgetevents' function. */
/* #undef HAVE_IO_PGETEVENTS */

/* Define to 1 if you have `io_set_eventfd' function. */
#define HAVE_IO_SET_EVENTFD 1

/* Define to 1 if you have the 'io_uring_enter' function. */
/* #undef HAVE_IO_URING_ENTER */

/* Define to 1 if you have the 'io_uring_register' function. */
/* #undef HAVE_IO_URING_REGISTER */

/* Define to 1 if you have the 'io_uring_setup' function. */
/* #undef HAVE_IO_URING_SETUP */

/* Define to 1 if you have the 'kcmp' function. */
/* #undef HAVE_KCMP */

/* Define to 1 if you have the <keyutils.h> header file. */
/* #undef HAVE_KEYUTILS_H */

/* Define to 1 if you have libacl and it's headers installed */
#define HAVE_LIBACL 1

/* Define to 1 if you have libaio and it's headers installed. */
#define HAVE_LIBAIO 1

/* Define to 1 if you have the <libaio.h> header file. */
#define HAVE_LIBAIO_H 1

/* Define to 1 if you have libcap-2 installed. */
#define HAVE_LIBCAP 1

/* Define whether libcrypto and openssl headers are installed */
#define HAVE_LIBCRYPTO 1

/* Define to 1 if you have libkeyutils installed. */
/* #undef HAVE_LIBKEYUTILS */

/* Define to 1 if you have libmnl library and headers */
/* #undef HAVE_LIBMNL */

/* Define to 1 if you have both SELinux libraries and headers. */
/* #undef HAVE_LIBSELINUX_DEVEL */

/* Define to 1 if you have the <linux/can.h> header file. */
#define HAVE_LINUX_CAN_H 1

/* Define to 1 if you have the <linux/cgroupstats.h> header file. */
#define HAVE_LINUX_CGROUPSTATS_H 1

/* Define to 1 if you have the <linux/close_range.h> header file. */
#define HAVE_LINUX_CLOSE_RANGE_H 1

/* Define to 1 if you have the <linux/cryptouser.h> header file. */
#define HAVE_LINUX_CRYPTOUSER_H 1

/* Define to 1 if you have the <linux/dccp.h> header file. */
#define HAVE_LINUX_DCCP_H 1

/* Define to 1 if you have the <linux/fsverity.h> header file. */
#define HAVE_LINUX_FSVERITY_H 1

/* Define to 1 if you have the <linux/fs.h> header file. */
#define HAVE_LINUX_FS_H 1

/* Define to 1 if you have the <linux/futex.h> header file. */
#define HAVE_LINUX_FUTEX_H 1

/* Define to 1 if you have the <linux/genetlink.h> header file. */
#define HAVE_LINUX_GENETLINK_H 1

/* Define to 1 if you have the <linux/if_alg.h> header file. */
#define HAVE_LINUX_IF_ALG_H 1

/* Define to 1 if you have the <linux/if_ether.h> header file. */
#define HAVE_LINUX_IF_ETHER_H 1

/* Define to 1 if you have the <linux/if_packet.h> header file. */
#define HAVE_LINUX_IF_PACKET_H 1

/* Define to 1 if you have the <linux/ioprio.h> header file. */
#define HAVE_LINUX_IOPRIO_H 1

/* Define to 1 if you have the <linux/io_uring.h> header file. */
#define HAVE_LINUX_IO_URING_H 1

/* Define to 1 if you have the <linux/keyctl.h> header file. */
#define HAVE_LINUX_KEYCTL_H 1

/* Define to 1 if you have the <linux/mempolicy.h> header file. */
#define HAVE_LINUX_MEMPOLICY_H 1

/* Define to 1 if you have the <linux/module.h> header file. */
#define HAVE_LINUX_MODULE_H 1

/* Define to 1 if you have the <linux/mount.h> header file. */
#define HAVE_LINUX_MOUNT_H 1

/* Define to 1 if you have the <linux/netlink.h> header file. */
#define HAVE_LINUX_NETLINK_H 1

/* Define to 1 if having a valid linux/random.h */
#define HAVE_LINUX_RANDOM_H 1

/* Define to 1 if you have the <linux/seccomp.h> header file. */
#define HAVE_LINUX_SECCOMP_H 1

/* Define to 1 if you have the <linux/securebits.h> header file. */
#define HAVE_LINUX_SECUREBITS_H 1

/* Define to 1 if you have the <linux/signalfd.h> header file. */
#define HAVE_LINUX_SIGNALFD_H 1

/* Define to 1 if you have the <linux/taskstats.h> header file. */
#define HAVE_LINUX_TASKSTATS_H 1

/* Define to 1 if you have the <linux/tls.h> header file. */
#define HAVE_LINUX_TLS_H 1

/* Define to 1 if you have the <linux/tty.h> header file. */
#define HAVE_LINUX_TTY_H 1

/* Define to 1 if you have the <linux/types.h> header file. */
#define HAVE_LINUX_TYPES_H 1

/* Define to 1 if you have the <linux/userfaultfd.h> header file. */
#define HAVE_LINUX_USERFAULTFD_H 1

/* Define to 1 if you have the <linux/vm_sockets.h> header file. */
#define HAVE_LINUX_VM_SOCKETS_H 1

/* Define to 1 if you have the 'mallinfo' function. */
#define HAVE_MALLINFO 1

/* Define to 1 if you have the 'mallinfo2' function. */
#define HAVE_MALLINFO2 1

/* Define to 1 if you have the 'mallopt' function. */
#define HAVE_MALLOPT 1

/* Define to 1 if you have the 'mkdtemp' function. */
#define HAVE_MKDTEMP 1

/* Define to 1 if you have the 'modify_ldt' function. */
/* #undef HAVE_MODIFY_LDT */

/* Define to 1 if you have the 'mount_setattr' function. */
#define HAVE_MOUNT_SETATTR 1

/* Define to 1 if you have the 'move_mount' function. */
#define HAVE_MOVE_MOUNT 1

/* Define to 1 if you have the 'name_to_handle_at' function. */
#define HAVE_NAME_TO_HANDLE_AT 1

/* Define to 1 if you have the <netinet/sctp.h> header file. */
/* #undef HAVE_NETINET_SCTP_H */

/* Define to 1 if you have newer libcap-2 installed. */
#define HAVE_NEWER_LIBCAP 1

/* Define to 1 if you have the <numaif.h> header file. */
/* #undef HAVE_NUMAIF_H */

/* Define to 1 if you have the <numa.h> header file. */
/* #undef HAVE_NUMA_H */

/* Define to 1 if you have libnuma and it's headers version >= 2 installed. */
/* #undef HAVE_NUMA_V2 */

/* Define to 1 if you have the 'openat' function. */
#define HAVE_OPENAT 1

/* Define to 1 if you have the 'openat2' function. */
/* #undef HAVE_OPENAT2 */

/* Define to 1 if you have the <openssl/sha.h> header file. */
#define HAVE_OPENSSL_SHA_H 1

/* Define to 1 if you have the 'open_tree' function. */
#define HAVE_OPEN_TREE 1

/* Define to 1 if you have the 'pidfd_getfd' function. */
#define HAVE_PIDFD_GETFD 1

/* Define to 1 if you have the 'pidfd_open' function. */
#define HAVE_PIDFD_OPEN 1

/* Define to 1 if you have the 'pidfd_send_signal' function. */
#define HAVE_PIDFD_SEND_SIGNAL 1

/* Define to 1 if you have the 'pkey_mprotect' function. */
#define HAVE_PKEY_MPROTECT 1

/* Define to 1 if you have the 'preadv' function. */
#define HAVE_PREADV 1

/* Define to 1 if you have the 'preadv2' function. */
#define HAVE_PREADV2 1

/* Define to 1 if you have the 'profil' function. */
#define HAVE_PROFIL 1

/* Define to 1 if you have the <pthread.h> header file. */
#define HAVE_PTHREAD_H 1

/* Define to 1 if you have the 'pwritev' function. */
#define HAVE_PWRITEV 1

/* Define to 1 if you have the 'pwritev2' function. */
#define HAVE_PWRITEV2 1

/* Define to 1 if you have the 'quotactl_fd' function. */
/* #undef HAVE_QUOTACTL_FD */

/* Define to 1 if you have the 'rand_r' function. */
#define HAVE_RAND_R 1

/* Define to 1 if you have the 'recvmmsg' function. */
#define HAVE_RECVMMSG 1

/* Define to 1 if you have the 'renameat2' function. */
#define HAVE_RENAMEAT2 1

/* Define to 1 if you have the 'sched_getcpu' function. */
#define HAVE_SCHED_GETCPU 1

/* Define to 1 if you have the <selinux/selinux.h> header file. */
/* #undef HAVE_SELINUX_SELINUX_H */

/* Define to 1 if you have the 'sendmmsg' function. */
#define HAVE_SENDMMSG 1

/* Define to 1 if you have the 'sethostid' function. */
#define HAVE_SETHOSTID 1

/* Define to 1 if you have the 'setns' function. */
#define HAVE_SETNS 1

/* Define to 1 if you have the 'signalfd' function. */
#define HAVE_SIGNALFD 1

/* Define to 1 if you have the 'sigpending' function. */
#define HAVE_SIGPENDING 1

/* Define to 1 if you have the 'splice' function. */
#define HAVE_SPLICE 1

/* Define to 1 if you have the 'statx' function. */
#define HAVE_STATX 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdio.h> header file. */
#define HAVE_STDIO_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the 'stime' function. */
/* #undef HAVE_STIME */

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if the system has the type 'struct acct_v3'. */
#define HAVE_STRUCT_ACCT_V3 1

/* Define to 1 if the system has the type 'struct af_alg_iv'. */
#define HAVE_STRUCT_AF_ALG_IV 1

/* Define to 1 if the system has the type 'struct fanotify_event_info_error'.
   */
#define HAVE_STRUCT_FANOTIFY_EVENT_INFO_ERROR 1

/* Define to 1 if the system has the type 'struct fanotify_event_info_fid'. */
#define HAVE_STRUCT_FANOTIFY_EVENT_INFO_FID 1

/* Define to 1 if 'fsid.__val' is a member of 'struct
   fanotify_event_info_fid'. */
/* #undef HAVE_STRUCT_FANOTIFY_EVENT_INFO_FID_FSID___VAL */

/* Define to 1 if the system has the type 'struct fanotify_event_info_header'.
   */
#define HAVE_STRUCT_FANOTIFY_EVENT_INFO_HEADER 1

/* Define to 1 if the system has the type 'struct fanotify_event_info_pidfd'.
   */
#define HAVE_STRUCT_FANOTIFY_EVENT_INFO_PIDFD 1

/* Define to 1 if the system has the type 'struct file_dedupe_range'. */
#define HAVE_STRUCT_FILE_DEDUPE_RANGE 1

/* Define to 1 if the system has the type 'struct file_handle'. */
#define HAVE_STRUCT_FILE_HANDLE 1

/* Define to 1 if the system has the type 'struct fsverity_enable_arg'. */
#define HAVE_STRUCT_FSVERITY_ENABLE_ARG 1

/* Define to 1 if the system has the type 'struct fs_quota_statv'. */
/* #undef HAVE_STRUCT_FS_QUOTA_STATV */

/* Define to 1 if the system has the type 'struct futex_waitv'. */
#define HAVE_STRUCT_FUTEX_WAITV 1

/* Define to 1 if you have struct f_owner_ex */
#define HAVE_STRUCT_F_OWNER_EX 1

/* Define to 1 if the system has the type 'struct if_nextdqblk'. */
#define HAVE_STRUCT_IF_NEXTDQBLK 1

/* Define to 1 if the system has the type 'struct iovec'. */
#define HAVE_STRUCT_IOVEC 1

/* Define to 1 if the system has the type 'struct ipc64_perm'. */
/* #undef HAVE_STRUCT_IPC64_PERM */

/* Define to 1 if the system has the type 'struct loop_config'. */
#define HAVE_STRUCT_LOOP_CONFIG 1

/* Define to 1 if the system has the type 'struct mmsghdr'. */
#define HAVE_STRUCT_MMSGHDR 1

/* Define to 1 if the system has the type 'struct modify_ldt_ldt_s'. */
/* #undef HAVE_STRUCT_MODIFY_LDT_LDT_S */

/* Define to 1 if the system has the type 'struct mount_attr'. */
#define HAVE_STRUCT_MOUNT_ATTR 1

/* Define to 1 if the system has the type 'struct msqid64_ds'. */
/* #undef HAVE_STRUCT_MSQID64_DS */

/* Define to 1 if 'aux_head' is a member of 'struct perf_event_mmap_page'. */
#define HAVE_STRUCT_PERF_EVENT_MMAP_PAGE_AUX_HEAD 1

/* Define to 1 if the system has the type 'struct rlimit64'. */
#define HAVE_STRUCT_RLIMIT64 1

/* Define to 1 if the system has the type 'struct semid64_ds'. */
/* #undef HAVE_STRUCT_SEMID64_DS */

/* Define to 1 if the system has the type 'struct shmid64_ds'. */
/* #undef HAVE_STRUCT_SHMID64_DS */

/* Define to 1 if 'sa_sigaction' is a member of 'struct sigaction'. */
#define HAVE_STRUCT_SIGACTION_SA_SIGACTION 1

/* Define to 1 if 'ssi_signo' is a member of 'struct signalfd_siginfo'. */
#define HAVE_STRUCT_SIGNALFD_SIGINFO_SSI_SIGNO 1

/* Define to 1 if the system has the type 'struct sockaddr_alg'. */
#define HAVE_STRUCT_SOCKADDR_ALG 1

/* Define to 1 if the system has the type 'struct statx'. */
#define HAVE_STRUCT_STATX 1

/* Define to 1 if 'stx_dio_mem_align' is a member of 'struct statx'. */
#define HAVE_STRUCT_STATX_STX_DIO_MEM_ALIGN 1

/* Define to 1 if 'stx_mnt_id' is a member of 'struct statx'. */
#define HAVE_STRUCT_STATX_STX_MNT_ID 1

/* Define to 1 if the system has the type 'struct statx_timestamp'. */
#define HAVE_STRUCT_STATX_TIMESTAMP 1

/* Define to 1 if 'freepages_count' is a member of 'struct taskstats'. */
#define HAVE_STRUCT_TASKSTATS_FREEPAGES_COUNT 1

/* Define to 1 if 'nvcsw' is a member of 'struct taskstats'. */
#define HAVE_STRUCT_TASKSTATS_NVCSW 1

/* Define to 1 if 'read_bytes' is a member of 'struct taskstats'. */
#define HAVE_STRUCT_TASKSTATS_READ_BYTES 1

/* Define to 1 if the system has the type 'struct termio'. */
#define HAVE_STRUCT_TERMIO 1

/* Define to 1 if the system has the type 'struct tpacket_req3'. */
#define HAVE_STRUCT_TPACKET_REQ3 1

/* Define to 1 if the system has the type 'struct user_desc'. */
/* #undef HAVE_STRUCT_USER_DESC */

/* Define to 1 if 'domainname' is a member of 'struct utsname'. */
#define HAVE_STRUCT_UTSNAME_DOMAINNAME 1

/* Define to 1 if the system has the type 'struct xt_entry_match'. */
#define HAVE_STRUCT_XT_ENTRY_MATCH 1

/* Define to 1 if the system has the type 'struct xt_entry_target'. */
#define HAVE_STRUCT_XT_ENTRY_TARGET 1

/* Define to 1 if the system has the type 'struct __kernel_itimerspec'. */
/* #undef HAVE_STRUCT___KERNEL_ITIMERSPEC */

/* Define to 1 if the system has the type 'struct __kernel_old_itimerspec'. */
/* #undef HAVE_STRUCT___KERNEL_OLD_ITIMERSPEC */

/* Define to 1 if the system has the type 'struct __kernel_old_timespec'. */
/* #undef HAVE_STRUCT___KERNEL_OLD_TIMESPEC */

/* Define to 1 if the system has the type 'struct __kernel_old_timeval'. */
/* #undef HAVE_STRUCT___KERNEL_OLD_TIMEVAL */

/* Define to 1 if the system has the type 'struct __kernel_timespec'. */
/* #undef HAVE_STRUCT___KERNEL_TIMESPEC */

/* Define to 1 if you have the 'syncfs' function. */
#define HAVE_SYNCFS 1

/* Define to 1 if you have __sync_add_and_fetch */
#define HAVE_SYNC_ADD_AND_FETCH 1

/* Define to 1 if you have the 'sync_file_range' function. */
#define HAVE_SYNC_FILE_RANGE 1

/* Define to 1 if you have the <sys/acl.h> header file. */
#define HAVE_SYS_ACL_H 1

/* Define to 1 if you have the <sys/capability.h> header file. */
#define HAVE_SYS_CAPABILITY_H 1

/* Define to 1 if you have the <sys/epoll.h> header file. */
#define HAVE_SYS_EPOLL_H 1

/* Define to 1 if you have the <sys/fanotify.h> header file. */
#define HAVE_SYS_FANOTIFY_H 1

/* Define to 1 if you have the <sys/inotify.h> header file. */
#define HAVE_SYS_INOTIFY_H 1

/* Define to 1 if you have the <sys/pidfd.h> header file. */
#define HAVE_SYS_PIDFD_H 1

/* Define to 1 if you have the <sys/prctl.h> header file. */
#define HAVE_SYS_PRCTL_H 1

/* Define to 1 if you have the <sys/shm.h> header file. */
#define HAVE_SYS_SHM_H 1

/* Define to 1 if you have the <sys/signalfd.h> header file. */
#define HAVE_SYS_SIGNALFD_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/timerfd.h> header file. */
#define HAVE_SYS_TIMERFD_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <sys/ustat.h> header file. */
/* #undef HAVE_SYS_USTAT_H */

/* Define to 1 if you have the <sys/utsname.h> header file. */
#define HAVE_SYS_UTSNAME_H 1

/* Define to 1 if you have the <sys/xattr.h> header file. */
#define HAVE_SYS_XATTR_H 1

/* Define to 1 if you have the 'tee' function. */
#define HAVE_TEE 1

/* Define to 1 if you have the 'timerfd_create' function. */
#define HAVE_TIMERFD_CREATE 1

/* Define to 1 if you have the 'timerfd_gettime' function. */
#define HAVE_TIMERFD_GETTIME 1

/* Define to 1 if you have the 'timerfd_settime' function. */
#define HAVE_TIMERFD_SETTIME 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to 1 if you have the 'unshare' function. */
#define HAVE_UNSHARE 1

/* Define to 1 if you have the 'ustat' function. */
/* #undef HAVE_USTAT */

/* Define to 1 if you have utimensat(2) */
#define HAVE_UTIMENSAT 1

/* Define to 1 if you have the 'vfork' function. */
#define HAVE_VFORK 1

/* Define to 1 if you have the 'vmsplice' function. */
#define HAVE_VMSPLICE 1

/* Define to 1 if you have the 'xdr_char' function. */
/* #undef HAVE_XDR_CHAR */

/* Define to 1 if you have the <xfs/xqm.h> header file. */
/* #undef HAVE_XFS_XQM_H */

/* Error message when no NUMA support */
#define NUMA_ERROR_MSG "NUMA support was disabled during build"

/* Name of package */
#define PACKAGE "ltp"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "ltp@lists.linux.it"

/* Define to the full name of this package. */
#define PACKAGE_NAME "ltp"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "ltp LTP_VERSION"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "ltp"

/* Define to the home page for this package. */
#define PACKAGE_URL ""

/* Define to the version of this package. */
#define PACKAGE_VERSION "LTP_VERSION"

/* Define to 1 if all of the C89 standard headers exist (not just the ones
   required in a freestanding environment). This macro is provided for
   backward compatibility; new code need not use it. */
#define STDC_HEADERS 1

/* Target is running Linux w/out an MMU */
/* #undef UCLINUX */

/* Version number of package */
#define VERSION "LTP_VERSION"
