#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sem_wait sem_wait_1-1.run-test sem_wait_1-2.run-test sem_wait_11-1.run-test sem_wait_12-1.run-test sem_wait_13-1.run-test sem_wait_3-1.run-test sem_wait_5-1.run-test sem_wait_7-1.run-test
