#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_rwlock_unlock pthread_rwlock_unlock_1-1.run-test pthread_rwlock_unlock_2-1.run-test pthread_rwlock_unlock_3-1.run-test pthread_rwlock_unlock_4-1.run-test pthread_rwlock_unlock_4-2.run-test
