#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutex_trylock pthread_mutex_trylock_1-1.run-test pthread_mutex_trylock_1-2.run-test pthread_mutex_trylock_2-1.run-test pthread_mutex_trylock_3-1.run-test pthread_mutex_trylock_4-1.run-test pthread_mutex_trylock_4-2.run-test pthread_mutex_trylock_4-3.run-test
