#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/shm_unlink shm_unlink_1-1.run-test shm_unlink_10-1.run-test shm_unlink_10-2.run-test shm_unlink_11-1.run-test shm_unlink_2-1.run-test shm_unlink_3-1.run-test shm_unlink_4-1.sh shm_unlink_5-1.run-test shm_unlink_6-1.run-test shm_unlink_7-1.sh shm_unlink_8-1.run-test shm_unlink_9-1.run-test
