#!/bin/sh
# Copyright (c) 2003, Intel Corporation. All rights reserved.
# Created by:  crystal.xiong REMOVE-THIS AT intel DOT com
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.


# Test on failure, no message will be removed from the queue,
# the funcion will return a value of -1.

# This is tested implicitly via assertion 13,14,15,16,17,18,19

echo "Tested implicitly via assertion 13,14,15,16,17,18,19.  See output for status"
exit 0
