#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/aio_write aio_write_1-1.run-test aio_write_1-2.run-test aio_write_2-1.run-test aio_write_3-1.run-test aio_write_5-1.run-test aio_write_6-1.run-test aio_write_7-1.run-test aio_write_8-1.run-test aio_write_8-2.run-test aio_write_9-1.run-test aio_write_9-2.run-test
