#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cond_timedwait pthread_cond_timedwait_1-1.run-test pthread_cond_timedwait_2-1.run-test pthread_cond_timedwait_2-2.run-test pthread_cond_timedwait_2-3.run-test pthread_cond_timedwait_2-4.run-test pthread_cond_timedwait_2-5.run-test pthread_cond_timedwait_2-6.run-test pthread_cond_timedwait_2-7.run-test pthread_cond_timedwait_3-1.run-test pthread_cond_timedwait_4-1.run-test pthread_cond_timedwait_4-2.run-test pthread_cond_timedwait_4-3.run-test
