#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_setschedparam pthread_setschedparam_1-1.run-test pthread_setschedparam_1-2.run-test pthread_setschedparam_4-1.run-test pthread_setschedparam_5-1.run-test
