#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_barrier_wait pthread_barrier_wait_1-1.run-test pthread_barrier_wait_2-1.run-test pthread_barrier_wait_3-1.run-test pthread_barrier_wait_3-2.run-test pthread_barrier_wait_4-1.sh pthread_barrier_wait_5-1.sh
