#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/clock_getres clock_getres_1-1.run-test clock_getres_2-1.sh clock_getres_3-1.run-test clock_getres_4-1.sh clock_getres_5-1.run-test clock_getres_6-1.run-test clock_getres_6-2.run-test clock_getres_7-1.run-test clock_getres_8-1.run-test
