#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setinheritsched pthread_attr_setinheritsched_1-1.run-test pthread_attr_setinheritsched_2-1.run-test pthread_attr_setinheritsched_2-2.run-test pthread_attr_setinheritsched_2-3.run-test pthread_attr_setinheritsched_2-4.run-test pthread_attr_setinheritsched_3-1.sh pthread_attr_setinheritsched_4-1.run-test
