#!/bin/sh
../../../../bin/run-tests.sh conformance/interfaces/timer_settime/speculative timer_settime_speculative_12-1.run-test timer_settime_speculative_12-2.run-test timer_settime_speculative_12-3.run-test
