#! /bin/sh
#
# The originating process might not receive notification of the completion of
# the requested priority change until the higher priority process has executed.
#
# This is tested implicitly via assertion 10.

echo "Tested implicitly via assertion 10."
exit 0
