#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sched_getparam sched_getparam_1-1.run-test sched_getparam_2-1.run-test sched_getparam_3-1.run-test sched_getparam_4-1.run-test sched_getparam_5-1.sh sched_getparam_6-1.run-test
