#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_barrierattr_getpshared pthread_barrierattr_getpshared_1-1.run-test pthread_barrierattr_getpshared_2-1.run-test pthread_barrierattr_getpshared_3-1.sh
