#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_setcancelstate pthread_setcancelstate_1-1.run-test pthread_setcancelstate_1-2.run-test pthread_setcancelstate_2-1.run-test pthread_setcancelstate_3-1.run-test
