#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_setprotocol pthread_mutexattr_setprotocol_1-1.run-test pthread_mutexattr_setprotocol_2-2.sh pthread_mutexattr_setprotocol_3-1.run-test pthread_mutexattr_setprotocol_3-2.run-test
