#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_getattr mq_getattr_1-1.sh mq_getattr_2-1.run-test mq_getattr_2-2.run-test mq_getattr_3-1.run-test mq_getattr_4-1.run-test mq_getattr_5-1.sh mq_getattr_6-1.sh
