#!/bin/sh
../../../../bin/run-tests.sh conformance/interfaces/timer_getoverrun/speculative timer_getoverrun_speculative_6-1.run-test timer_getoverrun_speculative_6-2.run-test timer_getoverrun_speculative_6-3.run-test
