#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sigaltstack sigaltstack_1-1.run-test sigaltstack_10-1.run-test sigaltstack_11-1.run-test sigaltstack_12-1.run-test sigaltstack_2-1.run-test sigaltstack_3-1.run-test sigaltstack_5-1.run-test sigaltstack_6-1.run-test sigaltstack_7-1.run-test sigaltstack_8-1.run-test sigaltstack_9-1.run-test
