#!/bin/sh
# Copyright (c) 2002, Intel Corporation. All rights reserved.
# Created by:  rolla.n.selbak REMOVE-THIS AT intel DOT com
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.

#   Test pthread_mutexattr_settype()

#   It returns 0 upon success.

# This is tested implicitly via assertion 2,3 and 4 of pthread_mutexattr_settype.

echo "Tested implicitly via assertions 2,3 and 4  of pthread_mutexattr_settype().  See output for status"
exit 0
