#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/timer_getoverrun timer_getoverrun_1-1.run-test timer_getoverrun_2-1.run-test timer_getoverrun_2-2.run-test timer_getoverrun_2-3.run-test timer_getoverrun_4-1.sh timer_getoverrun_5-1.sh
