#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_timedreceive mq_timedreceive_1-1.run-test mq_timedreceive_10-1.run-test mq_timedreceive_10-2.run-test mq_timedreceive_11-1.run-test mq_timedreceive_12-1.sh mq_timedreceive_13-1.run-test mq_timedreceive_14-1.run-test mq_timedreceive_15-1.run-test mq_timedreceive_16-1.sh mq_timedreceive_17-1.run-test mq_timedreceive_17-2.run-test mq_timedreceive_17-3.run-test mq_timedreceive_18-1.run-test mq_timedreceive_18-2.run-test mq_timedreceive_2-1.run-test mq_timedreceive_4-1.sh mq_timedreceive_5-1.run-test mq_timedreceive_5-2.run-test mq_timedreceive_5-3.run-test mq_timedreceive_7-1.run-test mq_timedreceive_8-1.run-test
