#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setstacksize pthread_attr_setstacksize_1-1.run-test pthread_attr_setstacksize_2-1.run-test pthread_attr_setstacksize_3-1.sh pthread_attr_setstacksize_4-1.run-test
