#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setstack pthread_attr_setstack_1-1.run-test pthread_attr_setstack_2-1.run-test pthread_attr_setstack_3-1.sh pthread_attr_setstack_4-1.run-test pthread_attr_setstack_5-1.sh pthread_attr_setstack_6-1.run-test pthread_attr_setstack_7-1.run-test
