#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutex_timedlock pthread_mutex_timedlock_1-1.run-test pthread_mutex_timedlock_2-1.run-test pthread_mutex_timedlock_4-1.run-test pthread_mutex_timedlock_5-1.run-test pthread_mutex_timedlock_5-2.run-test pthread_mutex_timedlock_5-3.run-test
