#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_setpshared pthread_mutexattr_setpshared_1-1.run-test pthread_mutexattr_setpshared_1-2.run-test pthread_mutexattr_setpshared_2-1.run-test pthread_mutexattr_setpshared_2-2.run-test pthread_mutexattr_setpshared_3-1.run-test pthread_mutexattr_setpshared_3-2.run-test
