#!/bin/sh
../../../../bin/run-tests.sh conformance/interfaces/timer_gettime/speculative timer_gettime_speculative_6-1.run-test timer_gettime_speculative_6-2.run-test timer_gettime_speculative_6-3.run-test
