#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/raise raise_1-1.run-test raise_1-2.run-test raise_10000-1.run-test raise_2-1.run-test raise_4-1.run-test raise_6-1.run-test raise_7-1.run-test
