#! /bin/sh
#
# Test that the appropriate privilege is required to lock process memory with
# mlock()
#
# This is tested implicitly via assertion 12.

echo "Tested implicitly via assertion 12."
exit 0
