#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cleanup_pop pthread_cleanup_pop_1-1.run-test pthread_cleanup_pop_1-2.run-test pthread_cleanup_pop_1-3.run-test pthread_cleanup_pop_2-1.sh
