#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/clock_gettime clock_gettime_1-1.run-test clock_gettime_1-2.run-test clock_gettime_2-1.run-test clock_gettime_3-1.run-test clock_gettime_4-1.run-test clock_gettime_6-1.sh clock_gettime_7-1.run-test clock_gettime_8-1.run-test clock_gettime_8-2.run-test
