#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_gettype pthread_mutexattr_gettype_1-1.run-test pthread_mutexattr_gettype_1-2.run-test pthread_mutexattr_gettype_1-3.run-test pthread_mutexattr_gettype_1-4.run-test pthread_mutexattr_gettype_1-5.run-test
