#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_spin_init pthread_spin_init_1-1.run-test pthread_spin_init_2-1.run-test pthread_spin_init_2-2.run-test pthread_spin_init_3-1.sh pthread_spin_init_4-1.run-test
