#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_kill pthread_kill_1-1.run-test pthread_kill_1-2.run-test pthread_kill_2-1.run-test pthread_kill_3-1.run-test pthread_kill_7-1.run-test pthread_kill_8-1.run-test
