#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_barrier_init pthread_barrier_init_1-1.run-test pthread_barrier_init_2-1.sh pthread_barrier_init_3-1.run-test pthread_barrier_init_4-1.run-test
