#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/nanosleep nanosleep_1-1.run-test nanosleep_1-2.run-test nanosleep_1-3.run-test nanosleep_10000-1.run-test nanosleep_2-1.run-test nanosleep_3-1.run-test nanosleep_3-2.run-test nanosleep_4-1.sh nanosleep_5-1.run-test nanosleep_5-2.run-test nanosleep_6-1.run-test nanosleep_7-1.run-test nanosleep_7-2.run-test
