#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_spin_unlock pthread_spin_unlock_1-1.run-test pthread_spin_unlock_1-2.run-test pthread_spin_unlock_2-1.sh pthread_spin_unlock_3-1.run-test
