#! /bin/sh
#
# Test that the sched_setscheduler() function is considered successful when it
# succeeds in setting the scheduling policy and scheduling parameters of the
# process specified by pid to the values specified by policy and the structure
# pointed to by param, respectively.
#
# This is tested implicitly via assertions 1 and 4.

echo "Tested implicitly via assertions 1 and 4."
exit 0
