#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_getprioceiling pthread_mutexattr_getprioceiling_1-1.run-test pthread_mutexattr_getprioceiling_1-2.run-test pthread_mutexattr_getprioceiling_2-1.sh pthread_mutexattr_getprioceiling_3-1.run-test
