#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setschedparam pthread_attr_setschedparam_1-1.run-test pthread_attr_setschedparam_1-2.run-test pthread_attr_setschedparam_1-3.run-test pthread_attr_setschedparam_1-4.run-test pthread_attr_setschedparam_2-1.sh
