#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/aio_cancel aio_cancel_1-1.run-test aio_cancel_10-1.run-test aio_cancel_2-1.run-test aio_cancel_2-2.run-test aio_cancel_3-1.run-test aio_cancel_4-1.run-test aio_cancel_5-1.run-test aio_cancel_6-1.run-test aio_cancel_7-1.run-test aio_cancel_8-1.run-test aio_cancel_9-1.run-test
