#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/timer_gettime timer_gettime_1-1.run-test timer_gettime_1-2.run-test timer_gettime_1-3.run-test timer_gettime_1-4.run-test timer_gettime_2-1.run-test timer_gettime_2-2.run-test timer_gettime_3-1.run-test timer_gettime_4-1.sh timer_gettime_5-1.sh
