#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sem_getvalue sem_getvalue_1-1.run-test sem_getvalue_2-1.run-test sem_getvalue_2-2.run-test sem_getvalue_4-1.run-test sem_getvalue_5-1.run-test
