#!/bin/sh
# Copyright (c) 2002, Intel Corporation. All rights reserved.
# Created by:  julie.n.fleischer REMOVE-THIS AT intel DOT com
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.
#
# Test that clock_nanosleep() with CLOCK_REALTIME == nanosleep().
#
# This is not necessarily something that can be tested; however, it could
# also be considered to be tested implicitly via the other clock_nanosleep()
# tests which use CLOCK_REALTIME.

echo "Tested implicitly via other tests.  See output for status."
exit 0
