#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sched_setscheduler sched_setscheduler_1-1.run-test sched_setscheduler_15-1.run-test sched_setscheduler_15-2.run-test sched_setscheduler_16-1.run-test sched_setscheduler_17-1.run-test sched_setscheduler_17-2.run-test sched_setscheduler_17-3.run-test sched_setscheduler_17-4.run-test sched_setscheduler_17-5.run-test sched_setscheduler_17-6.run-test sched_setscheduler_17-7.run-test sched_setscheduler_18-1.sh sched_setscheduler_19-1.run-test sched_setscheduler_19-2.run-test sched_setscheduler_19-3.run-test sched_setscheduler_19-4.run-test sched_setscheduler_19-5.run-test sched_setscheduler_20-1.run-test sched_setscheduler_21-1.run-test sched_setscheduler_22-1.run-test sched_setscheduler_22-2.run-test sched_setscheduler_3-1.sh sched_setscheduler_4-1.run-test sched_setscheduler_8-1.sh
