#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_settype pthread_mutexattr_settype_1-1.run-test pthread_mutexattr_settype_2-1.run-test pthread_mutexattr_settype_3-1.run-test pthread_mutexattr_settype_3-2.run-test pthread_mutexattr_settype_3-3.run-test pthread_mutexattr_settype_3-4.run-test pthread_mutexattr_settype_6-1.sh pthread_mutexattr_settype_7-1.run-test
