#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_barrierattr_setpshared pthread_barrierattr_setpshared_1-1.run-test pthread_barrierattr_setpshared_2-1.run-test pthread_barrierattr_setpshared_3-1.sh
