#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/aio_read aio_read_1-1.run-test aio_read_10-1.run-test aio_read_11-1.run-test aio_read_11-2.run-test aio_read_3-1.run-test aio_read_3-2.run-test aio_read_4-1.run-test aio_read_5-1.run-test aio_read_7-1.run-test aio_read_8-1.run-test aio_read_9-1.run-test
