#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_close mq_close_1-1.run-test mq_close_2-1.run-test mq_close_3-1.run-test mq_close_3-2.run-test mq_close_3-3.run-test mq_close_4-1.run-test
