#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sem_timedwait sem_timedwait_1-1.run-test sem_timedwait_10-1.run-test sem_timedwait_11-1.run-test sem_timedwait_2-1.run-test sem_timedwait_2-2.run-test sem_timedwait_3-1.run-test sem_timedwait_4-1.run-test sem_timedwait_6-1.run-test sem_timedwait_6-2.run-test sem_timedwait_7-1.run-test sem_timedwait_9-1.run-test
