#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutex_lock pthread_mutex_lock_1-1.run-test pthread_mutex_lock_2-1.run-test pthread_mutex_lock_3-1.run-test pthread_mutex_lock_4-1.run-test pthread_mutex_lock_5-1.run-test
