#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sched_get_priority_max sched_get_priority_max_1-1.run-test sched_get_priority_max_1-2.run-test sched_get_priority_max_1-3.run-test sched_get_priority_max_1-4.run-test sched_get_priority_max_2-1.run-test sched_get_priority_max_3-1.sh
