#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/shm_open shm_open_1-1.run-test shm_open_11-1.run-test shm_open_13-1.run-test shm_open_13-2.sh shm_open_14-1.sh shm_open_14-2.run-test shm_open_15-1.run-test shm_open_16-1.run-test shm_open_17-1.run-test shm_open_18-1.run-test shm_open_20-1.run-test shm_open_20-2.run-test shm_open_20-3.run-test shm_open_21-1.run-test shm_open_22-1.run-test shm_open_23-1.run-test shm_open_25-1.run-test shm_open_26-1.run-test shm_open_26-2.run-test shm_open_28-1.run-test shm_open_28-2.run-test shm_open_28-3.run-test shm_open_30-1.sh shm_open_31-1.sh shm_open_32-1.run-test shm_open_34-1.run-test shm_open_35-1.sh shm_open_37-1.run-test shm_open_38-1.run-test shm_open_39-1.run-test shm_open_39-2.run-test shm_open_4-1.sh shm_open_41-1.run-test shm_open_5-1.run-test shm_open_8-1.run-test
