#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_condattr_setclock pthread_condattr_setclock_1-1.run-test pthread_condattr_setclock_1-2.run-test pthread_condattr_setclock_1-3.run-test pthread_condattr_setclock_2-1.run-test
