#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cond_wait pthread_cond_wait_1-1.run-test pthread_cond_wait_2-1.run-test pthread_cond_wait_2-2.run-test pthread_cond_wait_2-3.run-test pthread_cond_wait_3-1.run-test pthread_cond_wait_4-1.run-test
