#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_rwlock_timedwrlock pthread_rwlock_timedwrlock_1-1.run-test pthread_rwlock_timedwrlock_2-1.run-test pthread_rwlock_timedwrlock_3-1.run-test pthread_rwlock_timedwrlock_4-1.sh pthread_rwlock_timedwrlock_5-1.run-test pthread_rwlock_timedwrlock_6-1.run-test pthread_rwlock_timedwrlock_6-2.run-test pthread_rwlock_timedwrlock_7-1.sh
