#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cond_init pthread_cond_init_1-1.run-test pthread_cond_init_2-1.run-test pthread_cond_init_3-1.run-test pthread_cond_init_4-1.run-test pthread_cond_init_4-3.run-test
