#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_once pthread_once_1-1.run-test pthread_once_1-2.run-test pthread_once_1-3.run-test pthread_once_2-1.run-test pthread_once_3-1.run-test pthread_once_6-1.run-test
