#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_send mq_send_1-1.run-test mq_send_10-1.run-test mq_send_11-1.run-test mq_send_11-2.run-test mq_send_12-1.run-test mq_send_13-1.run-test mq_send_14-1.run-test mq_send_2-1.run-test mq_send_3-1.run-test mq_send_3-2.run-test mq_send_4-1.run-test mq_send_4-2.run-test mq_send_4-3.run-test mq_send_5-1.run-test mq_send_5-2.run-test mq_send_7-1.run-test mq_send_8-1.run-test mq_send_9-1.run-test
