#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_receive mq_receive_1-1.run-test mq_receive_10-1.run-test mq_receive_11-1.run-test mq_receive_11-2.run-test mq_receive_12-1.run-test mq_receive_13-1.run-test mq_receive_2-1.run-test mq_receive_4-1.sh mq_receive_5-1.run-test mq_receive_7-1.run-test mq_receive_8-1.run-test mq_receive_9-1.sh
