#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cond_signal pthread_cond_signal_1-1.run-test pthread_cond_signal_1-2.run-test pthread_cond_signal_2-1.run-test pthread_cond_signal_2-2.run-test pthread_cond_signal_4-1.run-test pthread_cond_signal_4-2.run-test
