#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/timer_create timer_create_1-1.run-test timer_create_10-1.run-test timer_create_11-1.run-test timer_create_13-1.sh timer_create_14-1.sh timer_create_16-1.run-test timer_create_3-1.run-test timer_create_4-1.sh timer_create_6-1.sh timer_create_7-1.run-test timer_create_8-1.run-test timer_create_9-1.run-test
