#! /bin/sh
#
# Test that the munlockall() function return a value of -1 if it is not
# supported by the implementation.
#
# This is tested implicitly via assertion 5.

echo "Tested implicitly via assertion 5."
exit 0

