#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_key_delete pthread_key_delete_1-1.run-test pthread_key_delete_1-2.run-test pthread_key_delete_2-1.run-test pthread_key_delete_3-1.sh
