#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sigwait sigwait_1-1.run-test sigwait_2-1.run-test sigwait_3-1.run-test sigwait_4-1.run-test sigwait_6-1.run-test sigwait_6-2.run-test sigwait_7-1.run-test sigwait_8-1.run-test
