#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_cancel pthread_cancel_1-1.run-test pthread_cancel_1-2.run-test pthread_cancel_1-3.run-test pthread_cancel_2-1.run-test pthread_cancel_2-2.run-test pthread_cancel_2-3.run-test pthread_cancel_3-1.run-test pthread_cancel_4-1.run-test pthread_cancel_5-1.run-test
