#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sched_getscheduler sched_getscheduler_1-1.run-test sched_getscheduler_3-1.run-test sched_getscheduler_4-1.run-test sched_getscheduler_5-1.run-test sched_getscheduler_6-1.sh sched_getscheduler_7-1.run-test
