#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sigwaitinfo sigwaitinfo_1-1.run-test sigwaitinfo_2-1.run-test sigwaitinfo_3-1.run-test sigwaitinfo_5-1.run-test sigwaitinfo_6-1.run-test sigwaitinfo_7-1.run-test sigwaitinfo_8-1.run-test sigwaitinfo_9-1.run-test
