#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_notify mq_notify_1-1.run-test mq_notify_2-1.run-test mq_notify_3-1.run-test mq_notify_4-1.run-test mq_notify_5-1.run-test mq_notify_6-1.sh mq_notify_7-1.sh mq_notify_8-1.run-test mq_notify_9-1.run-test
