#!/bin/sh
# Copyright (c) 2002, Intel Corporation. All rights reserved.
# Created by:  julie.n.fleischer REMOVE-THIS AT intel DOT com
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.


# Test that clock_getres() supports a clock_id of CLOCK_REALTIME.
#
# This is tested implicitly via assertion 1.

echo "Tested implicitly via assertion 1.  See output for status."
exit 0
