#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_setattr mq_setattr_1-1.run-test mq_setattr_1-2.run-test mq_setattr_2-1.run-test mq_setattr_3-1.sh mq_setattr_4-1.sh mq_setattr_5-1.run-test
