#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_spin_trylock pthread_spin_trylock_1-1.run-test pthread_spin_trylock_2-1.sh pthread_spin_trylock_3-1.sh pthread_spin_trylock_4-1.run-test
