#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sched_get_priority_min sched_get_priority_min_1-1.run-test sched_get_priority_min_1-2.run-test sched_get_priority_min_1-3.run-test sched_get_priority_min_1-4.run-test sched_get_priority_min_2-1.run-test sched_get_priority_min_3-1.sh
