#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_rwlock_init pthread_rwlock_init_1-1.run-test pthread_rwlock_init_2-1.run-test pthread_rwlock_init_3-1.run-test pthread_rwlock_init_4-1.sh pthread_rwlock_init_6-1.run-test
