#!/bin/sh
../../bin/run-tests.sh functional/semaphores semaphores_sem_conpro.run-test semaphores_sem_lock.run-test semaphores_sem_philosopher.run-test semaphores_sem_readerwriter.run-test semaphores_sem_sleepingbarber.run-test
