#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_detach pthread_detach_1-1.run-test pthread_detach_1-2.run-test pthread_detach_2-1.run-test pthread_detach_2-2.run-test pthread_detach_3-1.run-test pthread_detach_4-1.run-test pthread_detach_4-2.run-test pthread_detach_4-3.run-test
