#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setschedpolicy pthread_attr_setschedpolicy_1-1.run-test pthread_attr_setschedpolicy_1-2.run-test pthread_attr_setschedpolicy_1-3.run-test pthread_attr_setschedpolicy_2-1.run-test pthread_attr_setschedpolicy_3-1.sh pthread_attr_setschedpolicy_4-1.run-test pthread_attr_setschedpolicy_5-1.run-test
