#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mmap mmap_1-1.run-test mmap_1-2.run-test mmap_10-1.run-test mmap_11-1.run-test mmap_11-2.run-test mmap_11-3.run-test mmap_11-4.run-test mmap_11-5.run-test mmap_11-6.run-test mmap_12-1.run-test mmap_13-1.run-test mmap_14-1.run-test mmap_16-1.sh mmap_17-1.sh mmap_18-1.run-test mmap_19-1.run-test mmap_20-1.sh mmap_21-1.run-test mmap_23-1.run-test mmap_24-1.run-test mmap_24-2.run-test mmap_27-1.run-test mmap_3-1.run-test mmap_31-1.run-test mmap_32-1.run-test mmap_5-1.run-test mmap_6-1.run-test mmap_6-2.run-test mmap_6-3.run-test mmap_6-4.run-test mmap_6-5.run-test mmap_6-6.run-test mmap_7-1.run-test mmap_7-2.run-test mmap_7-3.run-test mmap_7-4.run-test mmap_9-1.run-test
