#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutex_destroy pthread_mutex_destroy_1-1.run-test pthread_mutex_destroy_2-1.run-test pthread_mutex_destroy_2-2.run-test pthread_mutex_destroy_3-1.run-test pthread_mutex_destroy_5-1.run-test pthread_mutex_destroy_5-2.run-test
