#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_getdetachstate pthread_attr_getdetachstate_1-1.run-test pthread_attr_getdetachstate_1-2.run-test pthread_attr_getdetachstate_2-1.sh
