#! /bin/sh
#
# Test that sched_get_priority_max() sets errno == EINVAL if policy is not a
# valid policy.
# This is tested implicitly via assertion 2.

echo "Tested implicitly via assertion 2."
exit 0


