#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sched_rr_get_interval sched_rr_get_interval_1-1.run-test sched_rr_get_interval_2-1.run-test sched_rr_get_interval_3-1.run-test sched_rr_get_interval_4-1.sh
