#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_open mq_open_1-1.run-test mq_open_11-1.run-test mq_open_12-1.run-test mq_open_13-1.run-test mq_open_15-1.run-test mq_open_16-1.run-test mq_open_18-1.run-test mq_open_19-1.run-test mq_open_2-1.run-test mq_open_20-1.run-test mq_open_21-1.run-test mq_open_23-1.run-test mq_open_25-2.run-test mq_open_27-1.run-test mq_open_27-2.run-test mq_open_29-1.run-test mq_open_3-1.run-test mq_open_7-1.run-test mq_open_7-2.run-test mq_open_7-3.run-test mq_open_8-1.run-test mq_open_8-2.run-test mq_open_9-1.run-test mq_open_9-2.run-test
