#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/aio_fsync aio_fsync_12-1.run-test aio_fsync_14-1.run-test aio_fsync_2-1.run-test aio_fsync_3-1.run-test aio_fsync_4-1.run-test aio_fsync_5-1.run-test aio_fsync_8-1.run-test aio_fsync_8-2.run-test aio_fsync_8-3.run-test aio_fsync_8-4.run-test aio_fsync_9-1.run-test
