#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mlockall mlockall_13-1.run-test mlockall_13-2.run-test mlockall_15-1.run-test mlockall_2-1.sh mlockall_3-6.run-test mlockall_3-7.run-test mlockall_7-1.sh mlockall_8-1.run-test mlockall_9-1.sh
