#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sched_setparam sched_setparam_1-1.run-test sched_setparam_11-1.sh sched_setparam_2-1.run-test sched_setparam_2-2.run-test sched_setparam_20-1.run-test sched_setparam_21-1.run-test sched_setparam_21-2.run-test sched_setparam_22-1.run-test sched_setparam_23-1.run-test sched_setparam_23-2.run-test sched_setparam_23-3.run-test sched_setparam_23-4.run-test sched_setparam_23-5.run-test sched_setparam_23-6.run-test sched_setparam_23-7.run-test sched_setparam_24-1.sh sched_setparam_25-1.run-test sched_setparam_25-2.run-test sched_setparam_25-3.run-test sched_setparam_25-4.run-test sched_setparam_26-1.run-test sched_setparam_27-1.run-test sched_setparam_4-1.sh sched_setparam_5-1.run-test sched_setparam_9-1.run-test
