#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_setprioceiling pthread_mutexattr_setprioceiling_1-1.run-test pthread_mutexattr_setprioceiling_2-1.sh pthread_mutexattr_setprioceiling_3-1.run-test pthread_mutexattr_setprioceiling_3-2.run-test
