#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_atfork pthread_atfork_1-1.run-test pthread_atfork_1-2.run-test pthread_atfork_2-1.run-test pthread_atfork_2-2.run-test pthread_atfork_3-1.sh pthread_atfork_3-2.run-test pthread_atfork_3-3.run-test pthread_atfork_4-1.run-test
