#!/bin/sh

# Copyright (c) 2002, Intel Corporation. All rights reserved.
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.

# Test pthread_rwlock_timedrdlock()

#	Shall return zero if the lock for writing on the read-write lock object
#	referenced by rwlock is acquired. Otherwise, an error number shall be returned
#	to indicate the error

# This is tested implicitly via assertion 1.

echo "Tested implicitly via assertion 1.  See output for status"
exit 0
