#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_rwlock_rdlock pthread_rwlock_rdlock_1-1.run-test pthread_rwlock_rdlock_2-1.run-test pthread_rwlock_rdlock_2-2.run-test pthread_rwlock_rdlock_2-3.run-test pthread_rwlock_rdlock_4-1.run-test pthread_rwlock_rdlock_5-1.run-test pthread_rwlock_rdlock_6-2.sh
