#!/bin/sh
# Copyright (c) 2002, Intel Corporation. All rights reserved.
# Created by:  julie.n.fleischer REMOVE-THIS AT intel DOT com
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.


# Test that evp points to a sigevent structure that tells what notification
# will take place when the timer expires.

# This is tested implicitly via assertion 1.  A full test of the parameters
# of the sigevent structure is out of the scope of this test suite.

echo "Tested implicitly via assertion 1.  See output for status"
exit 0
