#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sem_unlink sem_unlink_1-1.run-test sem_unlink_2-1.run-test sem_unlink_2-2.run-test sem_unlink_3-1.run-test sem_unlink_4-1.run-test sem_unlink_4-2.run-test sem_unlink_5-1.run-test sem_unlink_6-1.run-test sem_unlink_7-1.run-test sem_unlink_9-1.run-test
