#! /bin/sh
#
# Test that the munlock() function return -1 upon unsuccessful completion.
#
# This is tested implicitly via assertions 10 and 11.

echo "Tested implicitly via assertions 10 and 11."
exit 0
