#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/clock_getcpuclockid clock_getcpuclockid_1-1.run-test clock_getcpuclockid_1-2.run-test clock_getcpuclockid_2-1.run-test clock_getcpuclockid_5-1.run-test clock_getcpuclockid_6-1.run-test
