#! /bin/sh
#
# Test that the shm_unlink() function return -1 upon unsuccessful completion.
#
# This is tested implicitly via assertions 9 to 11.

echo "Tested implicitly via assertions 9 to 11."
exit 0
