#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_attr_setscope pthread_attr_setscope_1-1.run-test pthread_attr_setscope_3-1.sh pthread_attr_setscope_4-1.run-test pthread_attr_setscope_5-1.run-test
