#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sigset sigset_1-1.run-test sigset_10-1.run-test sigset_2-1.run-test sigset_3-1.run-test sigset_4-1.run-test sigset_5-1.run-test sigset_6-1.run-test sigset_7-1.run-test sigset_8-1.run-test sigset_9-1.run-test
