#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_key_create pthread_key_create_1-1.run-test pthread_key_create_1-2.run-test pthread_key_create_2-1.run-test pthread_key_create_3-1.run-test
