#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sigqueue sigqueue_1-1.run-test sigqueue_10-1.run-test sigqueue_11-1.run-test sigqueue_12-1.run-test sigqueue_2-1.run-test sigqueue_2-2.run-test sigqueue_3-1.run-test sigqueue_4-1.run-test sigqueue_5-1.run-test sigqueue_6-1.run-test sigqueue_7-1.run-test sigqueue_8-1.run-test sigqueue_9-1.run-test
