#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_condattr_getpshared pthread_condattr_getpshared_1-1.run-test pthread_condattr_getpshared_1-2.run-test pthread_condattr_getpshared_2-1.run-test pthread_condattr_getpshared_3-1.sh
