#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_spin_lock pthread_spin_lock_1-1.run-test pthread_spin_lock_1-2.run-test pthread_spin_lock_2-1.sh pthread_spin_lock_3-1.run-test pthread_spin_lock_3-2.run-test
