#! /bin/sh
#
# Test that the scheduling policy and scheduling parameters are set for the
# process whose process ID is equal to pid when such a process exists and the
# the calling process has permission.
#
# This is tested implicitly via assertion 1.

echo "Tested implicitly via assertion 1."
exit 0
