#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutexattr_destroy pthread_mutexattr_destroy_1-1.run-test pthread_mutexattr_destroy_2-1.run-test pthread_mutexattr_destroy_3-1.run-test pthread_mutexattr_destroy_4-1.run-test
