#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sigtimedwait sigtimedwait_1-1.run-test sigtimedwait_2-1.run-test sigtimedwait_4-1.run-test sigtimedwait_5-1.run-test sigtimedwait_6-1.run-test
