#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sem_init sem_init_1-1.run-test sem_init_2-1.run-test sem_init_2-2.run-test sem_init_3-1.run-test sem_init_3-2.run-test sem_init_3-3.run-test sem_init_5-1.run-test sem_init_5-2.run-test sem_init_6-1.run-test sem_init_7-1.run-test
