#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_rwlockattr_getpshared pthread_rwlockattr_getpshared_1-1.run-test pthread_rwlockattr_getpshared_2-1.run-test pthread_rwlockattr_getpshared_4-1.run-test pthread_rwlockattr_getpshared_5-1.sh
