#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/clock_nanosleep clock_nanosleep_1-1.run-test clock_nanosleep_1-2.sh clock_nanosleep_1-3.run-test clock_nanosleep_1-4.run-test clock_nanosleep_1-5.run-test clock_nanosleep_10-1.run-test clock_nanosleep_11-1.run-test clock_nanosleep_13-1.run-test clock_nanosleep_15-1.sh clock_nanosleep_2-1.run-test clock_nanosleep_2-2.run-test clock_nanosleep_2-3.run-test clock_nanosleep_3-1.run-test clock_nanosleep_4-1.sh clock_nanosleep_5-1.sh clock_nanosleep_6-1.sh clock_nanosleep_8-1.sh clock_nanosleep_9-1.run-test
