#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/sigprocmask sigprocmask_10-1.run-test sigprocmask_12-1.run-test sigprocmask_15-1.run-test sigprocmask_17-1.run-test sigprocmask_4-1.run-test sigprocmask_5-1.run-test sigprocmask_6-1.run-test sigprocmask_7-1.run-test sigprocmask_8-1.run-test sigprocmask_8-2.run-test sigprocmask_8-3.run-test sigprocmask_9-1.run-test
