#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_rwlock_timedrdlock pthread_rwlock_timedrdlock_1-1.run-test pthread_rwlock_timedrdlock_2-1.run-test pthread_rwlock_timedrdlock_3-1.run-test pthread_rwlock_timedrdlock_4-1.sh pthread_rwlock_timedrdlock_5-1.run-test pthread_rwlock_timedrdlock_6-1.run-test pthread_rwlock_timedrdlock_6-2.run-test pthread_rwlock_timedrdlock_7-1.sh
