#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/killpg killpg_1-1.run-test killpg_1-2.run-test killpg_2-1.run-test killpg_4-1.run-test killpg_5-1.run-test killpg_6-1.run-test killpg_8-1.run-test
