#!/bin/sh
# Copyright (c) 2002, Intel Corporation. All rights reserved.
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.

#	If successful, the pthread_rwlockattr_setpshared( ) function shall
#	return zero; otherwise, an error number shall be returned to indicate
#	the error.
#

# This is tested implicitly via assertion 1.

echo "Tested implicitly via assertion 1.  See output for status"
exit 0
