#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/mq_timedsend mq_timedsend_1-1.run-test mq_timedsend_10-1.run-test mq_timedsend_11-1.run-test mq_timedsend_11-2.run-test mq_timedsend_12-1.run-test mq_timedsend_13-1.run-test mq_timedsend_14-1.run-test mq_timedsend_15-1.run-test mq_timedsend_16-1.run-test mq_timedsend_18-1.run-test mq_timedsend_19-1.run-test mq_timedsend_2-1.run-test mq_timedsend_20-1.run-test mq_timedsend_3-1.run-test mq_timedsend_3-2.run-test mq_timedsend_4-1.run-test mq_timedsend_4-2.run-test mq_timedsend_4-3.run-test mq_timedsend_5-1.run-test mq_timedsend_5-2.run-test mq_timedsend_5-3.run-test mq_timedsend_7-1.run-test mq_timedsend_8-1.run-test mq_timedsend_9-1.run-test
