#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutex_init pthread_mutex_init_1-1.run-test pthread_mutex_init_1-2.run-test pthread_mutex_init_2-1.run-test pthread_mutex_init_3-1.run-test pthread_mutex_init_3-2.run-test pthread_mutex_init_4-1.run-test pthread_mutex_init_5-1.run-test
