#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_mutex_getprioceiling pthread_mutex_getprioceiling_1-1.run-test pthread_mutex_getprioceiling_3-1.run-test pthread_mutex_getprioceiling_3-2.run-test pthread_mutex_getprioceiling_3-3.run-test
