#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_exit pthread_exit_1-1.run-test pthread_exit_1-2.run-test pthread_exit_2-1.run-test pthread_exit_2-2.run-test pthread_exit_3-1.run-test pthread_exit_3-2.run-test pthread_exit_4-1.run-test pthread_exit_5-1.run-test pthread_exit_6-1.run-test pthread_exit_6-2.run-test
