#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/timer_settime timer_settime_1-1.run-test timer_settime_1-2.run-test timer_settime_10-1.sh timer_settime_11-1.sh timer_settime_13-1.run-test timer_settime_2-1.run-test timer_settime_3-1.run-test timer_settime_3-2.run-test timer_settime_3-3.run-test timer_settime_4-1.sh timer_settime_5-1.run-test timer_settime_5-2.run-test timer_settime_5-3.run-test timer_settime_6-1.run-test timer_settime_8-1.run-test timer_settime_8-2.run-test timer_settime_8-3.run-test timer_settime_8-4.run-test timer_settime_9-1.run-test timer_settime_9-2.run-test
