#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_sigmask pthread_sigmask_10-1.run-test pthread_sigmask_12-1.run-test pthread_sigmask_14-1.run-test pthread_sigmask_15-1.run-test pthread_sigmask_16-1.run-test pthread_sigmask_18-1.run-test pthread_sigmask_4-1.run-test pthread_sigmask_5-1.run-test pthread_sigmask_6-1.run-test pthread_sigmask_7-1.run-test pthread_sigmask_8-1.run-test pthread_sigmask_8-2.run-test pthread_sigmask_8-3.run-test pthread_sigmask_9-1.run-test
