#!/bin/sh
../../../bin/run-tests.sh conformance/interfaces/pthread_create pthread_create_1-1.run-test pthread_create_1-2.run-test pthread_create_1-3.run-test pthread_create_1-5.run-test pthread_create_1-6.run-test pthread_create_11-1.run-test pthread_create_12-1.run-test pthread_create_14-1.run-test pthread_create_15-1.run-test pthread_create_2-1.run-test pthread_create_3-1.run-test pthread_create_3-2.run-test pthread_create_4-1.run-test pthread_create_5-1.run-test pthread_create_8-1.run-test
